#==============================================================================#
# Copyright (c) 2015 Franklin Sobrinho.                                        #
#                                                                              #
# Permission is hereby granted, free of charge, to any person obtaining        #
# a copy of this software and associated documentation files (the "Software"), #
# to deal in the Software without restriction, including without               #
# limitation the rights to use, copy, modify, merge, publish,                  #
# distribute, sublicense, and/or sell copies of the Software, and to           #
# permit persons to whom the Software is furnished to do so, subject to        #
# the following conditions:                                                    #
#                                                                              #
# The above copyright notice and this permission notice shall be               #
# included in all copies or substantial portions of the Software.              #
#                                                                              #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,              #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF           #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       #
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         #
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         #
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            #
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       #
#==============================================================================#

extends Node

class DirectoryUtilities extends Directory:

	# Get plugin folder path
	func get_data_dir(extra_file = ''):
		return "res://addons/add_primitives/src".plus_file(extra_file)

	func get_file_list(path, extension = ''):
		var list = []

		if dir_exists(path):
			open(path)

			list_dir_begin()

			var next = get_next()

			while next:
				if not current_is_dir():
					if extension != '' and next.extension() == extension:
						list.push_back(next)

					else:
						list.push_back(next)

				next = get_next()

			list_dir_end()

		return list

# End DirectoryUtilities

var current_module = ""

var plugin

var popup_menu
var mesh_dialog

var builder
var mesh_instance
var mymaterial

var primitives = {}
var modules = {}
var my_materials = {}

# Utilites
var Dir = DirectoryUtilities.new()
onready var g = get_node("/root/global")
# static func safe_call(object, method, args = []):
# 	if object and object.has_method(method):
# 		return object.callv(method, args)

# 	return null

func load_script_text(fn):
  # var fn = obj.get_script().get_path()
  # pp(fn)
  # g.report("fn", [fn], 0 , 1000, {"only":true})
  # var script = Script.new()
  var script = load(fn)
  var file = File.new()
  file.open( fn, file.READ)
  var text = file.get_as_text()
  # print(text)
  script.set_source_code(text)
  script.reload(true)
  # script.take_over_path(fn)
  # obj.set_script(script)
  return script

func runrun(gdmapf='t1.gd'):
	var path = Dir.get_data_dir('testproc')
	# var script = load(path.plus_file(gdmapf))
	# script.reload()
	var script = load_script_text(path.plus_file(gdmapf))
	var container = get_node("/root/world/env/tpcontainer")
	if get_node("/root/world/env/tpcontainer/map") != null:
		container.remove_child( get_node("/root/world/env/tpcontainer/map"))
	var map = Spatial.new()
	map.set_name("map")
	container.add_child(map)
	script.run(self)



func _create_primitive(name, params={}):
	params['my_materials'] = my_materials
	var node = get_node("/root/world/env/tpcontainer/map")
	# var edited_scene = get_tree().get_edited_scene_root()

	# # There's no node selected, and the scene is not empty
	# if not node and edited_scene:
	# 	return

	mesh_instance = MeshInstance.new()

	node.add_child( mesh_instance)
	mesh_instance.set_owner(node)
	var uniq = str(randi()%1000000)
	mesh_instance.set_name(name+uniq)

	var start = OS.get_ticks_msec()

	builder = primitives[name].new()
#	mymaterial = load('testproc/box_sides.tres')
#	var path = Dir.get_data_dir('testproc')
#	mymaterial = load(path.plus_file('box_sides.tres'))
#	mymaterial.set_fixed_flag(FixedMaterial.FLAG_USE_COLOR_ARRAY, true)
#	mymaterial.set_fixed_flag(FixedMaterial.FLAG_USE_ALPHA, true)
#	builder.set_material(mymaterial)
	for i in params:
		builder.set(i, params[i])

	builder.update()

	mesh_instance.set_mesh(builder.get_mesh())

func _init():
	init_my_materials()
	primitives.clear()

	# for c in popup_menu.get_children():
	# 	if c extends PopupMenu:
	# 		c.free()

	# var submenus = {}

	var path = Dir.get_data_dir('primitives')

	var scripts = Dir.get_file_list(path, 'gd')
	scripts.sort()

	for f_name in scripts:
		var script = load(path.plus_file(f_name))
		var name = script.get_name()

		if name == '':
			continue

		primitives[name] = script

func init_my_materials():
	print("init my_materials")
	my_materials.clear()
	var path = Dir.get_data_dir('testproc')
	var mats = Dir.get_file_list(path, 'tres')
	for m in mats:
		my_materials[m] = load(path.plus_file(m))
