tool
extends CanvasLayer

# Paths to nodes
onready var g = get_node("/root/global")
var path_to_rich_text_panel = "control/panel/rich_text_label"
var path_to_line_edit = "control/panel/line_edit"
var path_to_animation_player = "control/animation_player"
var history =[]
var hist_cursor = -1

# Nodes
onready var rich_text_panel_node = get_node(path_to_rich_text_panel)
onready var line_edit_node = get_node(path_to_line_edit)
onready var animation_player_node = get_node(path_to_animation_player)

onready var gsvar = get_node("/root/gamestate");
# onready var g = get_node("/root/global")

onready var sc2char_flip = {
	KEY_QUOTEDBL:'"' ,
	KEY_PARENLEFT: ')' ,
	KEY_BRACKETLEFT: ']',
	KEY_BRACELEFT: '}'
}

# Command List
const CommandList = preload("command_list.gd")
onready var command_list = CommandList.new(self)

# Key Input : Enter the keymap values!
export (String) var input_map_enter
export (String) var input_map_toggle

# Console status
var is_enabled = false

# Console signal
signal console_status


func _ready():
	# Setup processes
	set_process(false)
	set_process_input(true)

	# Add indentation and scrolling so you always see bottom line
	rich_text_panel_node.push_indent(1)
	rich_text_panel_node.set_scroll_follow(true)


func _process(delta):
	# Prevents removal of command line sign "$ "
	if(line_edit_node.get_text().length() <= 1):
		clear()

	# Enable focus after animation has been completed and enabled
	if(!animation_player_node.is_playing() && is_enabled):
		line_edit_node.set_focus_mode(line_edit_node.FOCUS_CLICK)
		line_edit_node.grab_focus()


func _input(event):
	# Toggle console
	if(event.type == InputEvent.KEY and event.pressed):
		g.report("evsc", [event, event.scancode, event.unicode, String(event)], 1 , 0, {"only":true})
	if(event.type == InputEvent.KEY and event.pressed and  event.control):
		var quickeys = {
#		KEY_1:"/t1()",
		# KEY_1:"/align_bot(0)",
		KEY_1:"/m1()",
#		KEY_2:"/t2()",
		KEY_2:"/align_bot(1)",
		KEY_3:"/t3()",
		KEY_T:"/tmp()",
		KEY_R:"/rec_re_throw()",
		# KEY_S:"/setup()",
		KEY_M:"/mc()",
		# KEY_P:"/pf()",
		KEY_P:"/p()",
#		KEY_P:"/runfp()",
		KEY_B:"/clipboard_console_exec()",
		# KEY_C:"/chat_toggle()",
		KEY_N:"/netem()",
		KEY_H:"/toggle_radar()",
		KEY_K:"/bot_shoot3()",
		# KEY_V:"."
		}
		if quickeys.has(event.scancode):
			process_command(quickeys[event.scancode])
	if(event.is_action_pressed("input_map_toggle")):
		toggle()
		if is_enabled:
#			Input.set_mouse_mode(Input.MOUSE_MODE_CAPTURED);
			Input.set_mouse_mode(Input.MOUSE_MODE_VISIBLE);
#			line_edit_node.release_focus()
		if is_enabled:
			gsvar.change_focus("C")
#			gsvar.focus.push_back("console")
		else:
			gsvar.restore_focus()
#			gsvar.focus.pop_back()

	# Processes user input in console
	if(is_enabled):
#		if (event.type == InputEvent.KEY and event.pressed):
#			print(event)

		if(event.type == InputEvent.KEY and event.pressed && [KEY_QUOTEDBL , KEY_PARENLEFT , KEY_BRACKETLEFT , KEY_BRACELEFT ].has(event.scancode)):
			line_edit_node.append_at_cursor( sc2char_flip[ event.scancode ]  )
			line_edit_node.set_cursor_pos(line_edit_node.get_cursor_pos() - 1)

		if(event.is_action_pressed("input_map_enter")):
			process_command(line_edit_node.get_text())

		if history.size() > 0:

			if(event.is_action_pressed("history_up")):
				var cur_h = history[hist_cursor]
				hist_cursor += 1
				hist_cursor = hist_cursor % history.size()
				print("history[hist_cursor]")
				print(history[hist_cursor])
				print(hist_cursor)
				while(hist_cursor < history.size()-1 and (history[hist_cursor] == "." or cur_h == history[hist_cursor])):
					hist_cursor += 1
					hist_cursor = hist_cursor % history.size()
				line_edit_node.set_text(history[hist_cursor])

			if(event.is_action_pressed("history_down")):
				hist_cursor -= 1
				hist_cursor = hist_cursor % history.size()
				line_edit_node.set_text(history[hist_cursor])
	else:
		if(event.is_action_pressed("console_rerun_last_command")):
			if history.size() > 0:
				# process_command(history[0])
				process_command(".")


func process_command(msg):
	# Write input to console
	write(msg)
	history.push_front(msg)
	hist_cursor = -1

	# Clean the string
	msg = msg.replace("$ ", "")
	msg = msg.strip_edges()

	# Run the command if it's not empty
	if(msg != ""):
		var returned_msg = command_list.execute(msg)

		if(str(returned_msg) != ""):
			write(str(returned_msg))
			# write(g.eval_context().pretty_str(returned_msg))


# Writes to console
func write(msg):
	rich_text_panel_node.add_text(msg + "\n")
	clear()



# Execute command directly from outside (without interacting with console directly)'
# Usually used to toggle something on and off via console (so that it knows what is going on)
func execute(cmd):
	command_list.execute(cmd)


# Clears the command line
func clear():
	line_edit_node.set_text("$ ")
	line_edit_node.set_cursor_pos(2)


# Toggles the console window
func toggle():
	if(!animation_player_node.is_playing()):
		is_enabled = !is_enabled
		set_process(is_enabled)

		# Remove line edit focus when disabling console
		if(!is_enabled):
			line_edit_node.set_focus_mode(line_edit_node.FOCUS_NONE)
			line_edit_node.grab_focus()

		# Animation
		if(is_enabled):
			animation_player_node.play("open_console")
		else:
			animation_player_node.play("close_console")

		# Notify others
		notify_others()


# Notifies other scripts that are connected to this, that console status has changed
func notify_others():
	emit_signal("console_status", is_enabled)
